package graphomat.graph;
import java.util.Hashtable;
import java.util.Enumeration;
import java.io.Serializable;
import graphomat.util.DLinkedList;

/**
 *Connector is instantiated with every Node. The class is private so the 
 *Data can't get inconsitent.
 */
class Connector implements Serializable{
	private class ConnectEnum implements Enumeration {
		private Enumeration enum;
		private Node n;
		public ConnectEnum(Enumeration enum, Node n) {
			this.enum = enum;
			this.n = n;
		}

		public boolean hasMoreElements()	{return enum.hasMoreElements();}
		public Object nextElement()	{return ((Edge)enum.nextElement()).otherThan(n);}
	}

  private Node n;
  private DLinkedList edgeList;
  private Hashtable connections;

  public Connector(Node n, DLinkedList edgeList, int hashSize){
    this.n = n;
    this.edgeList = edgeList;
    connections = new Hashtable(hashSize);
  }

  protected Hashtable getConnections()	{return connections;}
	protected DLinkedList getEdgeList() {return edgeList;}

	public void connect(Node n) {
		Edge e;

		if (!isConnected(n) && n != this.n) {
			e = new Edge(n,this.n);
			getEdgeList().insertElement(e);
			n.getConnector().getConnections().put(this.n,e);
			connections.put(n,e);
		}
	}
	public void disconnect(Node n) {
		Enumeration enum = getConnections().elements();
		Edge e;

		while (enum.hasMoreElements()) {
			e = (Edge) enum.nextElement();
			if (n == e.otherThan(this.n)) {
				getEdgeList().deleteElement(e);
				n.getConnector().getConnections().remove(this.n);
				connections.remove(n);
				break;
			}
		}
	}
	public Edge getEdge(Node n) {return (Edge)connections.get(n);}
	public boolean isConnected(Node n)	{return connections.containsKey(n);}
	public boolean anyConnections()	{return !connections.isEmpty();}
	public Enumeration getConnectEnum()	{return new ConnectEnum(connections.elements(),n);}
	public Enumeration getEdgeEnum()	{return edgeList.getEnum();}

}

