package graphomat.graph;
import graphomat.util.LazyDouble;
import java.io.Serializable;
import java.util.Observer;
import java.util.Observable;
import java.lang.Math.*;

/**
 *Data class for the Edges. Every Edge contains two Nodes.
 */
public class Edge extends LazyDouble implements Observer, Serializable{
	private Node n1;
	private Node n2;

	public Edge() {}

	public Edge(Node n1, Node n2) {
		this.n1 = n1;
		this.n2 = n2;
		n1.addObserver(this);
		n2.addObserver(this);
	}
	
	/**
	 *Returns first Node.
	 */
	public Node getn1()	{return n1;}
	/**
	 *Returns second Node.
	 */
	public Node getn2()	{return n2;}
	
	public void update(Observable n, Object arg) {
		if (n instanceof Node) {
			if (arg == Node.DELETED || arg == Node.MOVED) invalidate();
		}
	}
  /**
   *Lazy Evulation of Edge length.
   */
	public void recalculate() {
		setValue(Math.sqrt(Math.pow(n1.getx() - n2.getx(),2)
	                   + Math.pow(n1.gety() - n2.gety(),2)));
	}
	/**
	 *Return true is two Edges got one same Node.
	 */
	public boolean toucheEdge(Edge otherEdge) {
		return (n1 == otherEdge.getn1() || n2 == otherEdge.getn2() ||
		        n2 == otherEdge.getn1() || n1 == otherEdge.getn2());
	}
	/**
	 *Return true if both Edges got both the same Node.
	 */
	public boolean sameEdge(Edge otherEdge) {
		return (n1 == otherEdge.getn1() && n2 == otherEdge.getn2() ||
		        n2 == otherEdge.getn1() && n1 == otherEdge.getn2());
	}
	/**
	 *Return true if both Edges got both the same Node.
	 */
	public boolean sameEdge(Node n3, Node n4) {
		return (n1 == n3 && n2 == n4 ||
		        n2 == n4 && n1 == n3);
	}
	/**
	 *Returns distance between the Nodes.
	 */
	public double getLength() {return getValue();}
	
	/**
	 *Retruns the other Node than the given one.
	 */
	public Node otherThan(Node n) {return ((n1 == n)? n2 : n1);}
	public String toString()	{return n1.toSimpleString() + " -- " + n2.toSimpleString();}
}
