package graphomat.graph;
import java.io.Serializable;
import java.util.*;
import graphomat.util.*;

/**
 *Data class for a Node.
 */
public class Node extends ObservSerializable{
	public static final Object DELETED = new Object();
	public static final Object MOVED = new Object();
	public static final Object STATECHANGE = new Object();
	private double x, y;
	private double ax, ay;
	private Connector connector;
	private boolean notifyMove = true;
	private double ox, oy;
	private boolean selected = false;

	public Node() {}

	public Node(double x, double y) {
		this.x = x;
		this.y = y;
	}
	
	/**
	 *Switches selection State on if off and visa versa.
	 */
	public void swapSelect() {
		selected = (selected)?false:true;
		notifyObservers(STATECHANGE);
	}
	/**
	 *Sets if Node State for selection.
	 */
	public void setSelected(boolean b) {
		if (b != selected) {
			selected = b;
			notifyObservers(STATECHANGE);
		}
	}
	/**
	 *Return true if selected.
	 */
	public boolean isSelected()	{return selected;}
	/**
	 *Internal Messagedelivering System.
	 */
	public void notifyObservers(Object arg) {
		setChanged();
		super.notifyObservers(arg);
	}
	
	/**
	 *Sets a new value for the x-coordiante of the node);
	 */
	public void setx(double x) 	{this.x = x; if (getNotifyMove()) notifyObservers(MOVED);}
	
	/**
	 *Sets a new value for the y-coordiante of the node);
	 */
	public void sety(double y) 	{this.y = y; if (getNotifyMove()) notifyObservers(MOVED);}
	
	/**
	 *Add a value for the x-coordiante of the node);
	 */
	public void movex(double x)	{this.x += x; if (getNotifyMove()) notifyObservers(MOVED);}
	
	/**
	 *Add a value for the y-coordiante of the node);
	 */
	public void movey(double y)	{this.y += y; if (getNotifyMove()) notifyObservers(MOVED);}
	
	/**
	 *Sets a new values for the node.
	 */
	public void set(double x, double y) {
		this.x = x;
		this.y = y;
		if (getNotifyMove()) notifyObservers(MOVED);
	}

	/**
	 *Adds values for the node.
	 */
	public void move(double x, double y)	{
		this.x += x;
		this.y += y;
		if (getNotifyMove()) notifyObservers(MOVED);
	}
	/**
	 *Swaps it's location with another Node.
	 */
	public void swap(Node n) {
    double tx, ty;
    tx = this.x;
    ty = this.y;
    this.x = n.getx();
    this.y = n.gety();
    n.set(tx,ty);
    if (getNotifyMove()) notifyObservers(MOVED);
	}

	public void setNotifyMove(boolean b) {
		if ((b == true) && (notifyMove == false)) {
			if ((ox != x) || (oy != y))	notifyObservers(MOVED);
			notifyMove = true;
		} else if ((b == false) && (notifyMove == true)) {
			ox = x; oy = y;
			notifyMove = false;
		}
	}

	public boolean getNotifyMove() {return notifyMove;}
	
	/**
	 *Returns the x-coordiante of the node.
	 */
	public double getx() 	{return this.x;}
	
	/**
	 *Returns the y-coordiante of the node.
	 */
	public double gety() 	{return this.y;}
	/**
	 *Sets speed in x-axis.
	 */
	public void setax(double ax)	{this.ax = ax;}
	/**
	 *Sets speed in y-axis.
	 */
	public void setay(double ay)	{this.ay = ay;}
	/**
	 *gets speed in x-axis.
	 */
	public double getax()	{return ax;}
	/**
	 *Sets speed in y-axis.
	 */
	public double getay()	{return ay;}
	
	/**
	 *Tests if the other node stands at the same Place the node do.
	 */
	public boolean samePlace(Node otherNode) {
		return (this.x == otherNode.getx()) && (this.y == otherNode.gety());
	}
	
	/**
	 *Tests if the node stands at the given place.
	 */
	public boolean samePlace(double x, double y) {
		return (this.x == x) && (this.y == y);
	}
	
	protected Hashtable getConnections()	{return connector.getConnections();}
	protected void setConnector(Connector c)	{connector = c;}
	protected Connector getConnector()	{return connector;}
	
	/**
	 *Tests if the node is connectedto the requested node..
	 */
	public boolean isConnected(Node n)	{return connector.isConnected(n);}
	
	/**
	 *Tests if the node is connected.
	 */
	public boolean anyConnections()	{return connector.anyConnections();}
	
	/**
	 *Returns the <code>Edge</code> responsible for the connection between the two nodes.
	 */
	public Edge getEdge(Node n)	{return connector.getEdge(n);}
	
	/**
	 *Connects the node with the give node.
	 */
	public void connect(Node n)	{connector.connect(n);}
	
	/**
	 *Disconnects the node with the give node.
	 */
	public void disconnect(Node n) {connector.disconnect(n);}

	/**
	 *Clears all Connections.
	 */
	public void disconnectAll() {
		Enumeration enum = connector.getConnectEnum();

		while(enum.hasMoreElements()) {
			((Node)enum.nextElement()).disconnect(this);}
	}
	
	/**
	 *Returns <code>Enumeration</code> of all connected Nodes.
	 */
	public Enumeration getConnectEnum() {return connector.getConnectEnum();}
	/**
	 *Returns <code>Enumeration</code> of all connected Node's Edges.
	 */
	public Enumeration getEdgeEnum()	{return connector.getEdgeEnum();}
	/**
	 *Returns a String containing the x- and y-coordiantes of the node.
	 */
	public String toSimpleString() {return "(" + x + " , " + y + ")";}

	/**
	 *Returns a String with all connected nodes and the node itself.
	 */
	public String toString() 	{
		StringBuffer output = new StringBuffer(toSimpleString());
		if (connector.anyConnections()) {
			Enumeration enum = connector.getConnectEnum();

			output.append(":{");
			while(enum.hasMoreElements()) {
				output.append(((Node)enum.nextElement()).toSimpleString());
				if (enum.hasMoreElements()) output.append(", ");
			}
			output.append("}");
		}
		return output.toString();
	}
}

