package graphomat.gui;
import java.awt.*;
import java.awt.event.*;
import java.util.Enumeration;
import java.util.*;
import com.sun.java.swing.*;
import graphomat.graph.*;
/**
 *Basical Graphical Interface to Graph. Interprets Mousemanipulation and takes care of
 *all changes in the Graph. Repaints itself if Graph changed.
 */
public class JGraphCanvas extends JComponent implements Observer{
	private static final int NODERADIUS = 6; 
	private Graph g;
	private SubGraph sg;
	public SubGraph clip;
	private double ratioToCanvas, sGWidth, sGHeight, minX, minY;
	private double showPercent = 0.8;
	private int toShowX, toShowY, cWidth, cHeight;
	private Node lockedNode;
	private boolean drag = false,
		rect = false;
	private int rectX1, rectY1, rectX2, rectY2;
	private double lastx, lasty;
	private int mode = 1;
	private Node firstNode;
	
	public JGraphCanvas(Graph g) {
		this.g = g;
		clip = new SubGraph(g);
		sg = g.getSelection();
		g.addObserver(this);
		recalculate();
		addMouseListener(new MouseAdapter() {
			public void mousePressed(MouseEvent e) {onPressed(e.getX(),e.getY());}
			public void mouseReleased(MouseEvent e) {onReleased(e.getX(),e.getY());}
		});	
		addMouseMotionListener(new MouseMotionAdapter() {
			public void mouseDragged(MouseEvent e) {onDrag(e.getX(),e.getY());}
		});	
		addComponentListener(new ComponentAdapter() {
			public void componentResized(ComponentEvent e) {
				recalculate();}});	
	}
	
	/**Get Graph which is connected to this Interface.*/
	public Graph getGraph() {return g;}
	/**Set another Graph to the Interface.*/
	public void setGraph(Graph g) {
		this.g = g;
		sg = g.getSelection();
		clip = new SubGraph(g);
		g.addObserver(this);
		recalculate();
		repaint();	
	}
	/**
	 *Returns Manipulation mode:
	 *<UL><LI>1 - Edit Mode. Moving Nodes. Selecting and deselecting Nodes.
	 *<LI>2 - Add/Delete Node Mode. Adding or Deleting Nodes.
	 *<LI>3 - Add/Delete Edge Mode. Adding or Deleting Edges. Two selections are necessary.
	 *</UL>
	 */
	public int getMode() {return mode;}
	/**Set Manipulation mode.*/
	public void setMode(int x) {
		if (x != mode)
		mode = x;
		firstNode = null;
	}
	private Node findNode(int x, int y){
		Enumeration enum = g.getNodeEnum();
		Node n;
		while (enum.hasMoreElements()){
			n = (Node)enum.nextElement();
			if (Math.sqrt(Math.pow((calcXToCanvas(n.getx()) - x),2) + 
			              Math.pow((calcYToCanvas(n.gety()) - y),2)) < NODERADIUS) return n;
		}
		return null;  
	}
	
	public void onPressed(int x, int y)	{
		lockedNode = findNode(x,y);
		lastx = x/ratioToCanvas;
		lasty = y/ratioToCanvas;
		
		if (lockedNode == null) {
			rect = true;
			rectX1 = x; rectY1 = y;
			rectX2 = x; rectY2 = y;
		}
	}
	public void onReleased(int x, int y){
		Enumeration enum;
		Node n = findNode(x,y);
		
		if (mode == 1) {
  		if (n != null && !(drag)) {
  			if (clip.inGraph(n)) {clip.deleteNode(n);}
  			else {clip.addNode(n);}
  			n.swapSelect();
  		}
  		drag = false;
  		if (rect) {
  			rect = false;
  			selectRect();
  			repaint();
  		}
  	} else if (mode == 2) {
  	    if (n != null) {g.deleteNode(n);}
  	    else {g.addNode(new Node(calcXToGraph(x),calcYToGraph(y)));}
  	} else if (mode == 3) {
  		if (n != null) {
  			if (firstNode != null) {
  				if (!firstNode.isConnected(n)) {
    				firstNode.connect(n);
    				firstNode = null;
    			} else {
    				firstNode.disconnect(n);
    				firstNode = null;
          }	    			
  			}	else {
  				firstNode = n;
  				repaint();
  			}
  		}
  	}     
	}
	
	public void onDrag(int x, int y){
		double movex, movey;
		Enumeration enum;
		if (mode == 1) {
    		if (lockedNode != null) {
    			if (clip.inGraph(lockedNode)) {
    				movex = x/ratioToCanvas - lastx;
    				movey = y/ratioToCanvas - lasty;
    				enum = clip.getNodeEnum();
    				while (enum.hasMoreElements()){
    					((Node)enum.nextElement()).move(movex, movey);
    				}
    			} else {
    				lockedNode.set(calcXToGraph(x),calcYToGraph(y));
    			}
    			lastx = x/ratioToCanvas;
    			lasty = y/ratioToCanvas;
    			drag = true;
    		} else {
    			rectX2 = x; rectY2 = y;
    			repaint();
    		}
    	}  
	}
	
	public void putRect(Graphics gr) {
		int x, y, w, h;
		if (rectX1 > rectX2) {x = rectX2;	w = rectX1 - rectX2;
		} else {x = rectX1;	w = rectX2 - rectX1;}
		if (rectY1 > rectY2) {y = rectY2;	h = rectY1 - rectY2;
		} else {y = rectY1;	h = rectY2 - rectY1;}
		gr.setColor(Color.gray);
		gr.drawRect(x,y,w,h);
	}	
	
	private void selectRect(){
		Enumeration	enum = g.getNodeEnum();
		Node n;
		boolean b1, b2;
		int x, y;
		while (enum.hasMoreElements()) {
			b1 = false; b2 = false;
			n = (Node)enum.nextElement();
			x = calcXToCanvas(n.getx());
			y = calcYToCanvas(n.gety());
			if (rectX1 < rectX2) {
				if (x > rectX1 && x < rectX2) b1 = true;
			} else if (x > rectX2 && x < rectX1) b1 = true;
			if (rectY1 < rectY2) {
				if (y > rectY1 && y < rectY2) b2 = true;
			} else if (y > rectY2 && y < rectY1) b2 = true;
  		if (b1 && b2) 
				if (clip.inGraph(n)) {clip.deleteNode(n);}
  				else {clip.addNode(n);}
  		n.swapSelect();
		}
	}
	/**Selects all Nodes.*/
	public void selectAll(){
		Enumeration	enum = g.getNodeEnum();
		Node n;
		while (enum.hasMoreElements()){
			n = (Node)enum.nextElement();
			clip.addNode(n);
			n.setSelected(true);
		}
	}
	/**Deselects all Nodes.*/
	public void deselectAll(){
		Enumeration	enum = g.getNodeEnum();
		Node n;
		while (enum.hasMoreElements()){
			n = (Node)enum.nextElement();
			clip.deleteNode(n);
			n.setSelected(false);
		}
	}
	
	public void paint(Graphics gr) {
		Node n;
		Edge e;
		int x, y, x2, y2;
		if (rect) {
			putRect(gr);
		}
		Enumeration enum = g.getEdgeEnum();
		gr.setColor(Color.black);
		while (enum.hasMoreElements()) {
			e = (Edge) enum.nextElement();
			gr.drawLine(calcXToCanvas(e.getn1().getx()),
			            calcYToCanvas(e.getn1().gety()),
			            calcXToCanvas(e.getn2().getx()),
			            calcYToCanvas(e.getn2().gety()));		
		}
		enum = g.getNodeEnum();
		while (enum.hasMoreElements()) {
			n = (Node) enum.nextElement();
			if (clip.inGraph(n)) {gr.setColor(Color.darkGray);}
			else if (n == firstNode) {gr.setColor(Color.white);}
			else {gr.setColor(Color.lightGray);}
			x = calcXToCanvas(n.getx());
			y = calcYToCanvas(n.gety());
			gr.fillOval(x - NODERADIUS, y - NODERADIUS,
			            2 * NODERADIUS + 1, 2 * NODERADIUS);
			gr.setColor(Color.black);
			gr.drawOval(x - NODERADIUS, y - NODERADIUS,
			            2 * NODERADIUS, 2 * NODERADIUS - 1);
		}
	}
	
	public void recalculate() {
		Dimension d = getSize();
		double xRatio, yRatio;
		
		cWidth = (int) (d.width * showPercent);
		cHeight = (int) (d.height * showPercent);
		toShowX = (int) ((d.width - cWidth) / 2);
		toShowY = (int) ((d.height - cHeight) / 2);
		sGWidth = sg.getMaxX() - sg.getMinX();
		sGHeight = sg.getMaxY() - sg.getMinY();
		minX = sg.getMinX();
		minY = sg.getMinY();
		if (sGWidth == 0) sGWidth = 1;
		if (sGHeight == 0) sGHeight = 1;
		xRatio = cWidth / sGWidth;
		yRatio = cHeight / sGHeight;
		if (xRatio < yRatio) {
			ratioToCanvas = xRatio;
			toShowY += (int) (cHeight - (sGHeight * xRatio)) / 2;
		} else {
			ratioToCanvas = yRatio;
			toShowX += (int) (cWidth - (sGWidth * yRatio)) / 2;
		}
	}
	
	public int calcXToCanvas(double v) {return (int) ((v - minX) * ratioToCanvas) + toShowX;}
	public int calcYToCanvas(double v) {return (int) ((v - minY) * ratioToCanvas) + toShowY;}
	public double calcXToGraph(int v) {return (v - toShowX) / ratioToCanvas + minX;}
	public double calcYToGraph(int v) {return (v - toShowY) / ratioToCanvas + minY;}
	public void update(Observable n, Object arg) {
		if (n instanceof Graph) {
			if (arg == Graph.NODEADDED || arg == Graph.NODEMOVED ||
			    arg == Graph.NODEDELETED) {recalculate(); repaint();}
			else if (arg == Graph.NODESTATECHANGE || arg == Graph.EDGEADDED ||
			         arg == Graph.EDGEDELETED) {repaint();}
		}
	}
}
