package graphomat.util;
import java.util.Enumeration;
import java.util.Observable;
import java.io.Serializable;
/**
 *A simple implementation of a double linked list. Objects can be be stored and deleted. 
 *You have also the possibility to walk through the whole list be getting an 
 *<code>Enumeration</code> with <code>getEnum()</code>. Infact this is the only possibility
 *to read the lists objects.
 */
public class DLinkedList extends ObservSerializable{
	private class ListElement implements Serializable{
		private Object data;
		private ListElement last;
		private ListElement next;
		
		public ListElement() {}
		
		public ListElement(Object data, ListElement last, ListElement next) {
			this.data = data;
			this.last = last;
			this.next = next;
		}
		
		public void setData(Object data)	{this.data = data;}
		public void setLast(ListElement last)	{this.last = last;}
		public void setNext(ListElement next)	{this.next = next;}
		public Object	getData()	{return data;}
		public ListElement getLast()	{return last;}
		public ListElement getNext()	{return next;}
	}
	
	private class ListEnum implements Enumeration{
		private DLinkedList list;
		private ListElement listElement;

		public ListEnum(DLinkedList list) {
			this.list = list;
			this.listElement = list.head;
		}
		
		public boolean hasMoreElements()	{return listElement.getNext() != list.tail;}
		public Object nextElement() {
			listElement = listElement.getNext();
			return listElement.getData();
		}
	}
	
	public static final Object ADDED = new Object();
	public static final Object DELETED = new Object();
		
	private ListElement head = new ListElement();
	private ListElement tail = new ListElement();
	private int elementCount;
	
	public DLinkedList() {
		head.setNext(tail);
		head.setLast(head);
		head.setData("Head");
		tail.setNext(tail);
		tail.setLast(head);
		tail.setData("Tail");
	}
	
	/**
	 *Inserts a new object at the end of the list.
	 */
	public void insertElement(Object data) {
		ListElement insert = new ListElement();
		ListElement beforeLast = tail.getLast();
		
		beforeLast.setNext(insert);
		tail.setLast(insert);
		insert.setNext(tail);
		insert.setLast(beforeLast);
		insert.setData(data);
		elementCount++;
		notifyObservers(ADDED);	
	}
		
	private ListElement searchElement(Object data) {
		ListElement search = head;
		
		while (search.getNext() != tail && search.getData() != data ) 
			search = search.getNext();
		return search;
	}
	
	private void deleteListElement(ListElement del) {
		ListElement next = del.getNext();
		ListElement last = del.getLast();
		
		if (del != tail) {
			next.setLast(last);
			last.setNext(next);
			del = null;
			elementCount--;	
			notifyObservers(DELETED);	
		}
	}
	
	/**
	 *Deletes a object of the list. If the object can't be found the method ignores your request.
	 */	
	public void deleteElement(Object data) {
		ListElement search = searchElement(data);
		deleteListElement(search); 
	}
	
	/**
	 *Returns an <code>Enumeration</code> containing all elements.
	 */	
	public Enumeration getEnum()	{return new ListEnum(this);} 
	
	/**
	 *Returns the number of objects in the list.
	 */
	public int getElementCount()	{return elementCount;}
	
	/**
	 *Searches for the requested Element in the List.
	 */
	public boolean inList(Object data) {return !(searchElement(data) == tail);}

	public  void notifyObservers(Object arg) {
		setChanged();
		super.notifyObservers(arg);
	}

	/**
	 *Calls <code>toString()</code> on every Element of the list and returns it as a String.
	 */
	public String toString() {
		StringBuffer output = new StringBuffer(" ");
		Enumeration enum = getEnum();
		
		while (enum.hasMoreElements()) {
			output.append(enum.nextElement());
			if (enum.hasMoreElements()) output.append(", ");
		}
		return output.toString();
	}
}

