package graphomat.util;
import java.io.*;

abstract public class LazyDouble implements Serializable{
	private boolean valid = false;
	private double value;
	
	/**
	 *Stores a <code>double</code> value and makes the value accessible
	 *(valid).
	 */
	public void setValue(double value) {
		this.value = value;
		valid = true;
	}
	
	/**
	 *This method needs to be <code>extend</code>ed by a subclass.
	 *It will be called if a recalculation of the stored value is needed.
	 */
	abstract public void recalculate();
	
	/**
	 *Forces to call <code>recalculate</code> before the value can be 
	 *acessed.
	 */
	public void invalidate()	{valid = false;}
	
	/**
	 *Returns the current value, calls <code>recalculate</code> if it's 
	 *invalide.
	 */
	public double getValue() {
		if (!valid) recalculate();
		return value;
	}
}